;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xd3785f27a03219b0091cb49ce41b9609 . "./code.cg2js/cg-status-bar.fasl")
(#x2820a902672df741bbb80233dfbac237 . "./code.cg2js/cg-hotspot.fasl")
(#x655ebd74581345b45b78a511b6a7192e . "./code.cg2js/cg-toolbar.fasl")
(#x2ae62218bde14c0425d3e4b61bf8c60b . "./code.cg2js/cg-clipboard-stack.fasl")
(#x619edf0154a0eb23027c6ab861dee613 . "./code.cg2js/cg-tooltip.fasl")
(#x38d8217cb833ab281001a5a5501fc759 . "./code.cg2js/cg-calendar.fasl")
(#xc10425cc04cb30c3cc0131a1985a1604 . "./code.cg2js/cg-item-list.fasl")
(#x3e61996634e154565eeb58b374ef8eb5 . "./code.cg2js/cg-bitmap-stream.fasl")
(#x1b07a6907ffbd935eaa51498c6a27c5e . "./code.cg2js/cg-trackbar.fasl")
(#xd12d5eec58a7a50edaf78cdac2034274 . "./code.cg2js/cg-class-grid.fasl")
(#x6312c14a63b88533a689dc68f4aa0621 . "./code.cg2js/cg-dialog-item.fasl")
(#x651dffc3d6b5897bef44993f7e0e951c . "./code.cg2js/cg-comtab.fasl")
(#xe286298b0e44b81a86100206c3a60dbb . "./code.cg2js/cg-lisp-text.fasl")
(#xf3d8bfbee5ebc48e7b8be10d0ba39a89 . "./code.cg2js/cg-button.fasl")
(#x5f3ef161af22cb67fdef26f5bb7c5aa0 . "./code.cg2js/cg-printing.fasl")
(#xf2ea74fabf92acdba29f8ee46181c165 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x6fd50501f79a5f9bb551c4b464a319b6 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xfdbdc044498c7ace5699cd5b9d33d2e5 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x6e37ca9b76563ac28b42a5df34b10c76 . "./code.cg2js/cg-selected-object.fasl")
(#x9dc3a6ad4e9260fed71d5e014c380925 . "./code.cg2js/cg-text-or-combo.fasl")
(#xf270ee0c30e0f0a07f89396ff2d727a8 . "./code.cg2js/cg-graphics-context.fasl")
(#xb8cdee0d89b92ab05376088328f691e6 . "./code.cg2js/cg-edit-in-place.fasl")
(#x611bc40441f0812cffcd93413225ee00 . "./code.cg2js/cg-string-dialog.fasl")
(#x20434ddebf34f6e9a88baf92912d160a . "./code.cg2js/cg-list-view.fasl")
(#xc49a8435a01e0f93c00921eda4357f66 . "./code.cg2js/cg-intersect.fasl")
(#x8eaecc9f62140f889fcdc7b1d9e3bdde . "./code.cg2js/cg-text-widget.fasl")
(#xfa3e53672b3701c6482623a979ad590f . "./code.cg2js/cg-directory-dialog.fasl")
(#x6fc663bd6e0fe72524eb03272adc216f . "./code.cg2js/cg-file-dialog.fasl")
(#xdb6935dfb2dd55ef16c12e9840f72235 . "./code.cg2js/cg-dde-utils.fasl")
(#x39698b59334dadf5d283f456ad92c1e2 . "./code.cg2js/cg-nodes.fasl")
(#x55badb582ccfb4117f18161da561ec19 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x3956fa6d865376895096cb27ab12af48 . "./code.cg2js/cg-up-down-control.fasl")
(#xe847fb3850a5b1d42939fa096973e2a2 . "./code.cg2js/cg-get-position.fasl")
(#x4380e100226278cbb7f28188d6e1aa8a . "./code.cg2js/cg-toggling-widget.fasl")
(#xb7cd7b021881cc54595e1a178276737a . "./code.cg2js/cg-acache.fasl")
(#xc74cfcbdd25a8229895a6e35d62a5253 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x0a2ea105132941fb47eb56d573cbc86b . "./code.cg2js/cg-base.fasl")
(#x11a50ca9de146b701bf6af31eebf56e0 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x57d6156979bd96617d73254bbf4e7154 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x8bd227e4e7e99644a48b08949c3e7a18 . "./code.cg2js/cg-property.fasl")
(#x4a74a836bcf758651f80e0314f8e1da7 . "./code.cg2js/cg-editable-text.fasl")
(#x68039111b86af1b150a1a394e28d5ebf . "./code.cg2js/cg-lamp.fasl")
(#x79f555bc8437e3ad5b3e245b3ee19d08 . "./code.cg2js/cg-progress-indicator.fasl")
(#xc42aa4c7a1c208808c635391ddbdbec4 . "./code.cg2js/cg-chart-widget.fasl")
(#x3434b99f4ddb23223cf6928d7bce7f9b . "./code.cg2js/cg-common-control.fasl")
(#x29d8f95c9f87788aedff5ca0d338caa9 . "./code.cg2js/cg-font-dialog.fasl")
(#xeebffbc5931c79afabe6943a15032b9b . "./code.cg2js/cg-gesture-emulation.fasl")
(#x0f8c9a2519a1d6a184ec8b75c6c7da35 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x677ea7a268c84cb210487efa24c2b66d . "./code.cg2js/cg-os-window.fasl")
(#x58e439fef135eff0dfb23a5763d83642 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xfb99d4c25c39c46b2c7b8c05186609b6 . "./code.cg2js/cg-pixmap.fasl")
(#x4b42eca943fd044d9df150d1b15c2916 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x37f529c6be07cd9a66648375e514592b . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x624422b5f5edbdb035f17f24f477ce21 . "./code.cg2js/cg-scaling-stream.fasl")
(#x61dcaa49d4ad739c816f4fd00944b95b . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xcf0966fe786fc826fa6108502ac5b2c7 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x398314c73743f6af756f33555c9f6d4e . "./code.cg2js/cg-timer.fasl")
(#x6741980230d5d531f6c1e8188aeb19be . "./code.cg2js/cg-scroll-bar.fasl")
(#xdc0f97cd0e3d347092e8e060d7c593c6 . "./code.cg2js/cg-caret.fasl")
(#x49f1d5d251c6a25a34e8cdb7522edea2 . "./code.cg2js/cg-template-string.fasl")
(#x9d71a0a5b070df89bcb7924256215c01 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xe64b4f0855906ee3ed11d4a62a2bd63e . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xa29ed30daff8448829ed804b519ed31f . "./code.cg2js/cg-combo-box.fasl")
(#x81274fdd2589f74b555bb1be1f88cf7b . "./code.cg2js/cg-text-edit-pane.fasl")
(#xc51c0fb9d1be791d76138bd9c4c0f64f . "./code.cg2js/cg-multi-picture-button.fasl")
(#x21ee31cb138b7759eb3cebb4db634edb . "./code.cg2js/cg-menu-tooltip.fasl")
(#xa691aaeb63fb2e42534f77830bd6f714 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x40b17ae16f4ca72c483e72ffc3b4e8d9 . "./code.cg2js/cg-player.fasl")
(#xd02789822a57b76e6990a71424afaecc . "./code.cg2js/cg-outline.fasl")
(#x56761b7efcc3b9dc91ea19d9a5c4293c . "./code.cg2js/cg-class-support.fasl")
(#x9448be813a3e5794b1ecf2697c14d97e . "./code.cg2js/cg-lettered-menu.fasl")
(#x3af9718c3f8c65eef4277fb74a475b27 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xb3b4c5bad564934139641340dbf1fcbc . "./code.cg2js/cg-fill-texture.fasl")
(#x9c58131c9a2a20ceb2e67bbb5013deeb . "./code.cg2js/cg-lisp-widget.fasl")
(#x23297103d58457f418c21f7df07df7a4 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x04f8206d8c6dbac7468f39094db6a0be . "./code.cg2js/cg-clipboard.fasl")
(#x26762bfa12386c00ae347d1f348d5c74 . "./code.cg2js/cg-wrap-string.fasl")
(#xc36283dcad6469f960099d297502e982 . "./code.cg2js/cg-os-widget.fasl")
(#x4cc78135b14b381b4f1a9bb9456edcbf . "./code.cg2js/cg-check-box.fasl")
(#xbb8117814a87c7d8ae7e02c79fcf8482 . "./code.cg2js/cg-choice-list.fasl")
(#xe63bdbf1f12363fe2e5bb4fb352f5483 . "./code.cg2js/cg-layout.fasl")
(#x688055b2df59cf7ea841ecdfd4f4f85f . "./code.cg2js/cg-group-box.fasl")
(#x406efbbc0b5a82e7ab066dbcc66ecb34 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xf3a0c191e642d299a032ce3573e012b5 . "./code.cg2js/cg-radio-button.fasl")
(#x58d739f9b056ad13949973b4b9bc571f . "./code.cg2js/cg-tab-control.fasl")
(#xd6b2645fee320b6d5b7e472c7b480acc . "./code.cg2js/cg.fasl")
(#x716dfc8efe44e5febdf7ad2f021d10d2 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x0e6697244e7b845a9fe196571d9ff710 . "./code.cg2js/cg-dropping-outline.fasl")
(#x15967b148c63c02eb3b06345a10fa4a2 . "./code.cg2js/cg-get-pixmap.fasl")
(#x8c3481e47955d613247fd0b709d9e48c . "./code.cg2js/cg-grid-widget.fasl")
(#x562359b88a9072fcec8346a587b3250a . "./code.cg2js/cg-utility-dialog.fasl")
(#x88a049ae90bb016640e2b7aaceb17828 . "./code.cg2js/cg-static-text.fasl")
(#x00a6c4a545cf1ef506b95261f401fa08 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x11066778d8f2e095c0c044b964c5a9f3 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x027b7f414af8ef95961801f9c5e528b5 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x3d1cb61fc4bbdf537b1a1332e0d10464 . "./code.cg2js/cg-bitmap-pane.fasl")
(#xb34547e43f9fd16b529d465942b1adda . "./code.cg2js/cg-plot-widget.fasl")
(#x077cbb7ea96fbbf1d4ada43f8a78b31f . "./code.cg2js/cg-color-dialog.fasl")
(#x5cf97bd254e3eb92f94a8ca3800b7781 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x9e31a10ced572a8076b917879f0299d4 . "./code.cg2js/cg-icon.fasl")
(#x69ee8c6b3dbef8c3d86e1def388f657f . "./code.cg2js/cg-dev.fasl")
(#x4a827e241f525c4884d4f391e99bbbd9 . "./code.cg2js/cg-choose-printer.fasl")
(#x1f2ba3e36721e4d74ef8555daf5a8c29 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x806440b9624d3d020eee764872ad61d0 . "./code.cg2js/cg-menu.fasl")
(#xb9614d662608363a6be18bb7d1110190 . "./code.cg2js/cg-yes-no-list.fasl")
(#xe3186f8c3167ee8bad55e8790b381f20 . "./code.cg2js/cg-palette.fasl")
(#x513ee6ee245aab05f482c960bedc8186 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xc117fd80af7d5045cde8e2917e441507 . "./code.cg2js/cg-message-dialog.fasl")
(#x2ceda8317d8b573dc288a9bdc69cf774 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xeb9e5f947279e3094b6c74895fb33f26 . "./code.cg2js/cg-split-bar.fasl")
(#xc4f5c4dfedaef48387a8cca5f12be27d . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x92865841323a0790bc1fddf41d9eee50 . "./code.cg2js/cg-web-browser.fasl")
(#xd47be4e3001302a0f78e8a46d6be38ed . "./code.cg2js/cg-project-window.fasl")
(#xf1943589e7c2538e30d56f3f1f16ac10 . "./code.cg2js/cg-picture-widget.fasl")
(#xce33f0bc0b8f58f49b12595da506714e . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x71a9f9011ffa5eb18ea31a6bd887628a . "./code.cg2js/cg-object-editor.fasl")
(#x002b463543b5ca67f2eef0dc0bd694b0 . "./code.cg2js/cg-paren-matching.fasl")
(#x1079597664ded3929b9cc38c56780b8b . "./code.cg2js/cg-sample-file-menu.fasl")
(#x074cdcde25e5836317208817dd06f01e . "./code.cg2js/cg-drawable.fasl")
(#xf78af8c4e9d122179a8125447f3baeb5 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xef742989df40d0b369074409b5f7267c . "./code.cg2js/cg-html-widget.fasl")
(#xf78cd27e212129e68d64cd31630019c9 . "./code.cg2js/cg-curve.fasl")
(#x38781e50c1429f91163106fc135acf9c . "./code.cg2js/cg-yes-no-string.fasl")
(#xb2dba25ca3fd5e5ee00cb31dcc66fb43 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x15ff4486b8c2ef49e3a154874249bf58 . "./code.cg2js/cg-html-dialog.fasl")
(#xa812c79ba40921bb360e826bb06262d7 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x2ad7d22f732edf678de50905f291261f . "./code.cg2js/cg-tray.fasl")
(#xa6dfdb20186b620ee42bbd5fb058c888 . "./code.idejs/ide-apropos.fasl")
(#xaeb002d73eaad83c107c59c1c29d9723 . "./code.idejs/ide-window-list.fasl")
(#x842bc1e37719fd9e2f768c7493fa9394 . "./code.idejs/ide-class-browser.fasl")
(#x126caf55972869b7a64b4f3c591960ba . "./code.idejs/ide-base.fasl")
(#xdf2de1431f38cdbbfcdd9358a1d89230 . "./code.idejs/ide-process-browser.fasl")
(#xe4684dc670414cc70747793c15a5945d . "./code.idejs/ide-acache.fasl")
(#xee0b74f317a26d9d84fab16c825f3960 . "./code.idejs/ide-navigator.fasl")
(#xd5009bd2716356ff1e602bd855fd6aac . "./code.idejs/ide-trace.fasl")
(#xab3a2e21aa7f3450be26a0c2bb3392c8 . "./code.idejs/ide-project.fasl")
(#x32cb8039f3d5c3e6bdf2504ddd40ddf9 . "./code.idejs/ide-patch-dialog.fasl")
(#x50dbc369ecb36482d33c7ca161c3c692 . "./code.idejs/ide-grapher.fasl")
(#x6b8657026b1f9a1860c573bf8b685d68 . "./code.idejs/ide-dev.fasl")
(#x51eb7fbc0e969c160e206419d99f739c . "./code.idejs/ide-keyshort.fasl")
(#x147ca0bd6599d01af6a97bc8ba0043a1 . "./code.idejs/ide-editor.fasl")
(#x466bb65ae08b480122c4ff55e6ee3e4c . "./code.idejs/ide-inspector.fasl")
(#x46a385fad39289863cb6f5135749a180 . "./code.idejs/ide.fasl")
(#xd7d8e6ba5f8eda9c7d06dc165f2509e8 . "./code.idejs/ide-clipboard.fasl")
(#xaae6222f3f9a219b0194446aaee6d385 . "./code.idejs/ide-find-definitions.fasl")
(#xe30ae1eee19bc2c9656c314bc20342b2 . "./code.idejs/ide-form.fasl")
(#x958f53c1e3bb112c3e5332e95f737a61 . "./code.idejs/ide-intro.fasl")
(#x37397907e6ad206b04a39a560aad065d . "./code.idejs/ide-find-in-files.fasl")
(#xa07b460e473f81132d6ad53537781153 . "./code.idejs/ide-package-list.fasl")
(#xd9ba162a49785026e0fa3cea4c759c5d . "./code.idejs/ide-class-grapher.fasl")
(#xf3f59eb0bf604c0b34476c9897472f9c . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
